@@ -1,5 +1,5 @@
 /*
- *  CU sudo version 1.5.3
+ *  CU sudo version 1.5.7
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *
  *  Please send bugs, changes, problems to sudo-bugs@courtesan.com
  *
- *  $Id: compat.h,v 1.28 1996/11/14 02:37:16 millert Exp $
+ *  $Id: compat.h,v 1.42 1998/10/21 23:50:10 millert Exp $
  */
 
 #ifndef _SUDO_COMPAT_H
@@ -84,6 +84,13 @@
 #  define S_IRWXU		0000700		/* rwx for owner */
 #endif /* S_IRWXU */
 
+/*
+ * Some OS's may not have this.
+ */
+#ifndef howmany
+#define howmany(x, y)	(((x) + ((y) - 1)) / (y))
+#endif
+
 /*
  * We need to know how long the longest password may be.
  * For alternate password schemes we need longer passwords.
@@ -93,7 +100,7 @@
 #  undef _PASSWD_LEN
 #  define _PASSWD_LEN		256
 #else
-#  if (SHADOW_TYPE == SPW_SECUREWARE)
+#  ifdef  HAVE_GETPRPWNAM
 #    undef _PASSWD_LEN
 #    define _PASSWD_LEN		AUTH_MAX_PASSWD_LENGTH
 #  else
@@ -108,8 +115,8 @@
 #        endif /* SHADOW_TYPE != SPW_NONE */
 #      endif /* PASS_MAX */
 #    endif /* !_PASSWD_LEN */
-#  endif /* HAVE_KERB4 || HAVE_AFS || HAVE_DCE || HAVE_SKEY || HAVE_OPIE */
-#endif /* SPW_SECUREWARE */
+#  endif /* HAVE_GETPRPWNAM */
+#endif /* HAVE_KERB4 || HAVE_AFS || HAVE_DCE || HAVE_SKEY || HAVE_OPIE */
 
 /*
  * Some OS's lack these
@@ -138,7 +145,7 @@
 #  ifdef __hpux
 #    define seteuid(_EUID)	(setresuid(UID_NO_CHANGE, _EUID, UID_NO_CHANGE))
 #  else
-#    define seteuid(_EUID)	(setresuid(UID_NO_CHANGE, _EUID))
+#    define seteuid(_EUID)	(setreuid(UID_NO_CHANGE, _EUID))
 #  endif /* __hpux */
 #endif /* HAVE_SETEUID */
 
