@@ -1,5 +1,5 @@
 /*
- * CU sudo version 1.5.4 (based on Root Group sudo version 1.1)
+ * CU sudo version 1.5.7 (based on Root Group sudo version 1.1)
  *
  * This software comes with no waranty whatsoever, use at your own risk.
  *
@@ -36,10 +36,6 @@
  *  Jeff Nieusma   Thu Mar 21 23:39:04 MST 1991
  */
 
-#ifndef lint
-static char rcsid[] = "$Id: logging.c,v 1.88 1998/01/13 04:48:42 millert Exp $";
-#endif /* lint */
-
 #include "config.h"
 
 #include <stdio.h>
@@ -60,16 +56,19 @@ static char rcsid[] = "$Id: logging.c,v 1.88 1998/01/13 04:48:42 millert Exp $";
 #endif /* HAVE_MALLOC_H && !STDC_HEADERS */
 #include <pwd.h>
 #include <signal.h>
+#include <time.h>
+#include <errno.h>
 #include <sys/types.h>
-#include <sys/time.h>
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
-#include <sys/errno.h>
 #include <netinet/in.h>
 
 #include "sudo.h"
-#include <options.h>
+
+#ifndef lint
+static const char rcsid[] = "$Id: logging.c,v 1.106 1998/11/18 04:16:13 millert Exp $";
+#endif /* lint */
 
 /*
  * Prototypes for local functions
@@ -87,12 +86,6 @@ static void syslog_wrapper	__P((int, char *, char *, char *));
 static char *logline;
 extern int errorlineno;
 
-/*
- * length of syslog-like header info used for mail and file logs
- * is len("Mon MM HH:MM:SS : username : ")
- */
-#define LOG_HEADER_LEN		29
-
 #ifdef BROKEN_SYSLOG
 #define MAXSYSLOGTRIES		16	/* num of retries for broken syslogs */
 #define SYSLOG(a,b,c,d)		syslog_wrapper(a,b,c,d)
@@ -136,7 +129,7 @@ void log_error(code)
     int code;
 {
     char *p;
-    int count;
+    int count, header_length;
     time_t now;
 #if (LOGGING & SLOG_FILE)
     mode_t oldmask;
@@ -147,17 +140,26 @@ void log_error(code)
     char *tmp, save;
 #endif /* LOGGING & SLOG_SYSLOG */
 
+    /*
+     * length of syslog-like header info used for mail and file logs
+     * is len("DDD MM HH:MM:SS : username : ") with an additional
+     * len("HOST=hostname : ") if HOST_IN_LOG is defined.
+     */
+    header_length = 21 + strlen(user_name);
+#ifdef HOST_IN_LOG
+    header_length += 8 + strlen(shost);
+#endif
+
     /*
      * Allocate enough memory for logline so we won't overflow it
      */
-    count = LOG_HEADER_LEN + 136 + 2 * MAXPATHLEN + strlen(tty) + strlen(cwd) +
+    count = header_length + 136 + 2 * MAXPATHLEN + strlen(tty) + strlen(cwd) +
 	    strlen(runas_user);
     if (cmnd_args)
 	count += strlen(cmnd_args);
 
     logline = (char *) malloc(count);
     if (logline == NULL) {
-	perror("malloc");
 	(void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
 	exit(1);
     }
@@ -168,12 +170,16 @@ void log_error(code)
      */
     now = time((time_t) 0);
     p = ctime(&now) + 4;
-    (void) sprintf(logline, "%15.15s : %8.8s : ", p, user_name);
+#ifdef HOST_IN_LOG
+    (void) sprintf(logline, "%15.15s : %s : HOST=%s : ", p, user_name, shost);
+#else
+    (void) sprintf(logline, "%15.15s : %s : ", p, user_name);
+#endif
 
     /*
      * we need a pointer to the end of logline for cheap appends.
      */
-    p = logline + LOG_HEADER_LEN;
+    p = logline + header_length;
 
     switch (code) {
 
@@ -276,8 +282,14 @@ void log_error(code)
 		tty, cwd, runas_user);
 	    break;
 
+	case BAD_ALLOCATION:
+	    (void) sprintf(p,
+		"allocation failure; TTY=%s ; PWD=%s ; USER=%s ; COMMAND=",
+		tty, cwd, runas_user);
+	    break;
+
 	default:
-	    strcat(p, "found a wierd error : ");
+	    strcat(p, "found a weird error : ");
 	    break;
     }
 
@@ -316,7 +328,7 @@ void log_error(code)
     /*
      * Log the full line, breaking into multiple syslog(3) calls if necesary
      */
-    p = &logline[LOG_HEADER_LEN];	/* skip past the date and user */
+    p = &logline[header_length];	/* skip past the date, host, and user */
     for (count = 0; count < strlen(logline) / MAXSYSLOGLEN + 1; count++) {
 	if (strlen(p) > MAXSYSLOGLEN) {
 	    /*
@@ -366,6 +378,9 @@ void log_error(code)
 	char *beg, *oldend, *end;
 	int maxlen = MAXLOGFILELEN;
 
+#ifndef WRAP_LOG
+       (void) fprintf(fp, "%s\n", logline);
+#else
 	/*
 	 * Print out logline with word wrap
 	 */
@@ -413,6 +428,7 @@ void log_error(code)
 		beg = NULL;			/* exit condition */
 	    }
 	}
+#endif
 
 	(void) fclose(fp);
     }
@@ -428,7 +444,7 @@ void log_error(code)
 
 
 
-#ifdef MAILER
+#ifdef _PATH_SENDMAIL
 /**********************************************************************
  *
  *  send_mail()
@@ -441,7 +457,7 @@ static char *mail_argv[] = { "sendmail", "-t", (char *) NULL };
 
 static void send_mail()
 {
-    char *mailer = MAILER;
+    char *mailer = _PATH_SENDMAIL;
     char *subject = MAILSUBJECT;
     int fd[2];
     char *p;
@@ -522,7 +538,7 @@ static void send_mail()
     /* no mailer defined */
     return;
 }
-#endif /* MAILER */
+#endif /* _PATH_SENDMAIL */
 
 
 
@@ -536,9 +552,15 @@ static void send_mail()
 static RETSIGTYPE reapchild(sig)
     int sig;
 {
-    int save_errno = errno;
+    int pid, status, save_errno = errno;
 
+#ifdef sudo_waitpid
+    do {
+	pid = sudo_waitpid(-1, &status, WNOHANG);
+    } while (pid == -1);
+#else
     (void) wait(NULL);
+#endif
 #ifndef POSIX_SIGNALS
     (void) signal(SIGCHLD, reapchild);
 #endif /* POSIX_SIGNALS */
@@ -658,9 +680,14 @@ void inform_user(code)
 		"Your timestamp file has a preposterous date, ignoring.\n");
 	    break;
 
+	case BAD_ALLOCATION:
+	    (void) fprintf(stderr,
+		"Resource allocation failure.\n");
+	    break;
+
 	default:
 	    (void) fprintf(stderr,
-		"Something wierd happened.\n\n");
+		"Something weird happened.\n\n");
 	    break;
     }
 }
@@ -715,6 +742,7 @@ static int appropriate(code)
     case SPOOF_ATTEMPT:
     case BAD_STAMPDIR:
     case BAD_STAMPFILE:
+    case BAD_ALLOCATION:
     default:
 	return (1);
 	break;
