@@ -1,5 +1,5 @@
 /*
- *  CU sudo version 1.5.4
+ *  CU sudo version 1.5.7
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,10 +24,6 @@
  * Chris Jepeway <jepeway@cs.utk.edu>
  */
 
-#ifndef lint
-static char rcsid[] = "$Id: parse.c,v 1.80 1998/01/13 15:32:41 millert Exp $";
-#endif /* lint */
-
 #include "config.h"
 
 #include <stdio.h>
@@ -83,7 +79,10 @@ static char rcsid[] = "$Id: parse.c,v 1.80 1998/01/13 15:32:41 millert Exp $";
 #endif
 
 #include "sudo.h"
-#include <options.h>
+
+#ifndef lint
+static const char rcsid[] = "$Id: parse.c,v 1.97 1998/11/18 04:16:13 millert Exp $";
+#endif /* lint */
 
 /*
  * Globals
@@ -105,17 +104,13 @@ static int has_meta	__P((char *));
 int validate(check_cmnd)
     int check_cmnd;
 {
-    FILE *sudoers_fp;
     int return_code;
 
     /* become sudoers file owner */
     set_perms(PERM_SUDOERS, 0);
 
-    if ((sudoers_fp = fopen(_PATH_SUDO_SUDOERS, "r")) == NULL) {
-	perror(_PATH_SUDO_SUDOERS);
-	log_error(NO_SUDOERS_FILE);
-	exit(1);
-    }
+    /* we opened _PATH_SUDO_SUDOERS in check_sudoers() so just rewind it */
+    rewind(sudoers_fp);
     yyin = sudoers_fp;
     yyout = stdout;
 
@@ -134,6 +129,7 @@ int validate(check_cmnd)
      * Don't need to keep this open...
      */
     (void) fclose(sudoers_fp);
+    sudoers_fp = NULL;
 
     /* relinquish extra privs */
     set_perms(PERM_USER, 0);
@@ -160,12 +156,13 @@ int validate(check_cmnd)
      */
     if (check_cmnd == FALSE)
 	while (top) {
-	    if (host_matches == TRUE)
+	    if (host_matches == TRUE) {
 		/* user may always do validate or list on allowed hosts */
 		if (no_passwd == TRUE)
 		    return(VALIDATE_OK_NOPASS);
 		else
 		    return(VALIDATE_OK);
+	    }
 	    top--;
 	}
     else
@@ -214,13 +211,15 @@ int command_matches(cmnd, user_args, path, sudoers_args)
     struct stat pst;
     DIR *dirp;
     struct dirent *dent;
-    char buf[MAXPATHLEN+1];
+    char buf[MAXPATHLEN];
     static char *c;
 
     /* don't bother with pseudo commands like "validate" */
     if (strchr(cmnd, '/') == NULL)
 	return(FALSE);
 
+    plen = strlen(path);
+
     /* only need to stat cmnd once since it never changes */
     if (cmnd_st.st_dev == 0) {
 	if (stat(cmnd, &cmnd_st) < 0)
@@ -252,9 +251,7 @@ int command_matches(cmnd, user_args, path, sudoers_args)
 	else
 	    return(FALSE);
     } else {
-	plen = strlen(path);
 	if (path[plen - 1] != '/') {
-#ifdef FAST_MATCH
 	    char *p;
 
 	    /* Only proceed if the basenames of cmnd and path are the same */
@@ -264,7 +261,6 @@ int command_matches(cmnd, user_args, path, sudoers_args)
 		p++;
 	    if (strcmp(c, p))
 		return(FALSE);
-#endif /* FAST_MATCH */
 
 	    if (stat(path, &pst) < 0)
 		return(FALSE);
@@ -294,13 +290,15 @@ int command_matches(cmnd, user_args, path, sudoers_args)
 	    return(FALSE);
 
 	while ((dent = readdir(dirp)) != NULL) {
+	    /* ignore paths > MAXPATHLEN (XXX - log) */
+	    if (plen + NAMLEN(dent) >= sizeof(buf))
+		continue;
 	    strcpy(buf, path);
 	    strcat(buf, dent->d_name);
-#ifdef FAST_MATCH
+
 	    /* only stat if basenames are not the same */
 	    if (strcmp(c, dent->d_name))
 		continue;
-#endif /* FAST_MATCH */
 	    if (stat(buf, &pst) < 0)
 		continue;
 	    if (cmnd_st.st_dev == pst.st_dev && cmnd_st.st_ino == pst.st_ino)
@@ -358,23 +356,27 @@ int usergr_matches(group, user)
     char *group;
     char *user;
 {
-    struct group *grpent;
+    struct group *grp;
+    struct passwd *pw;
     char **cur;
 
     /* make sure we have a valid usergroup, sudo style */
     if (*group++ != '%')
 	return(FALSE);
 
-    if ((grpent = getgrnam(group)) == NULL) 
+    if ((grp = getgrnam(group)) == NULL) 
 	return(FALSE);
 
     /*
      * Check against user's real gid as well as group's user list
      */
-    if (grpent->gr_gid == user_gid)
+    if ((pw = getpwnam(user)) == NULL)
+	return(FALSE);
+
+    if (grp->gr_gid == pw->pw_gid)
 	return(TRUE);
 
-    for (cur=grpent->gr_mem; *cur; cur++) {
+    for (cur=grp->gr_mem; *cur; cur++) {
 	if (strcmp(*cur, user) == 0)
 	    return(TRUE);
     }
@@ -407,13 +409,12 @@ int netgr_matches(netgr, host, user)
 #ifdef HAVE_GETDOMAINNAME
     /* get the domain name (if any) */
     if (domain == (char *) -1) {
-	if ((domain = (char *) malloc(MAXHOSTNAMELEN + 1)) == NULL) {
-	    perror("malloc");
+	if ((domain = (char *) malloc(MAXHOSTNAMELEN)) == NULL) {
 	    (void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
 	    exit(1);
 	}
 
-	if (getdomainname(domain, MAXHOSTNAMELEN + 1) != 0 || *domain == '\0') {
+	if (getdomainname(domain, MAXHOSTNAMELEN) != 0 || *domain == '\0') {
 	    (void) free(domain);
 	    domain = NULL;
 	}
