@@ -1,5 +1,5 @@
 /*
- * CU sudo version 1.5.3 (based on Root Group sudo version 1.1)
+ * CU sudo version 1.5.7 (based on Root Group sudo version 1.1)
  *
  * This software comes with no waranty whatsoever, use at your own risk.
  *
@@ -50,10 +50,6 @@
  *		Todd Miller  <Todd.Miller@courtesan.com>
  */
 
-#ifndef lint
-static char rcsid[] = "$Id: sudo.c,v 1.174 1996/11/14 02:37:16 millert Exp $";
-#endif /* lint */
-
 #define MAIN
 
 #include "config.h"
@@ -79,24 +75,14 @@ static char rcsid[] = "$Id: sudo.c,v 1.174 1996/11/14 02:37:16 millert Exp $";
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/time.h>
 #include <sys/param.h>
 #include <netinet/in.h>
 #include <netdb.h>
-#if (SHADOW_TYPE == SPW_SECUREWARE)
-#  ifdef __hpux
-#    include <hpsecurity.h>
-#  else
-#    include <sys/security.h>
-#  endif /* __hpux */
-#  include <prot.h>
-#endif /* SPW_SECUREWARE */
 #ifdef HAVE_DCE
 #include <pthread.h>
 #endif /* HAVE_DCE */
 
 #include "sudo.h"
-#include <options.h>
 #include "version.h"
 
 #ifndef STDC_HEADERS
@@ -109,6 +95,10 @@ extern char *strdup	__P((const char *));
 extern char *getenv	__P((char *));
 #endif /* STDC_HEADERS */
 
+#ifndef lint
+static const char rcsid[] = "$Id: sudo.c,v 1.213 1998/11/18 04:16:13 millert Exp $";
+#endif /* lint */
+
 
 /*
  * Local type declarations
@@ -126,7 +116,7 @@ static int  parse_args			__P((void));
 static void usage			__P((int));
 static void load_globals		__P((int));
 static int check_sudoers		__P((void));
-static void load_cmnd			__P((int));
+static int load_cmnd			__P((int));
 static void add_env			__P((int));
 static void clean_env			__P((char **, struct env_table *));
 extern int  user_is_exempt		__P((void));
@@ -141,14 +131,15 @@ char **Argv;
 int NewArgc = 0;
 char **NewArgv = NULL;
 struct passwd *user_pw_ent;
-char *runas_user = "root";
+char *runas_user = RUNAS_DEFAULT;
 char *cmnd = NULL;
 char *cmnd_args = NULL;
-char *tty = NULL;
+char *tty = "unknown";
 char *prompt;
-char host[MAXHOSTNAMELEN + 1];
+char host[MAXHOSTNAMELEN];
 char *shost;
-char cwd[MAXPATHLEN + 1];
+char cwd[MAXPATHLEN];
+FILE *sudoers_fp = NULL;
 struct stat cmnd_st;
 static char *runas_homedir = NULL;
 extern struct interface *interfaces;
@@ -161,7 +152,7 @@ extern int printmatches;
 struct env_table badenv_table[] = {
     { "IFS=", 4 },
     { "LD_", 3 },
-    { "_RLD_", 5 },
+    { "_RLD", 4 },
 #ifdef __hpux
     { "SHLIB_PATH=", 11 },
 #endif /* __hpux */
@@ -188,13 +179,16 @@ int main(argc, argv)
     int argc;
     char **argv;
 {
-    int rtn;
+    int rtn, cmnd_status = FOUND;
     int sudo_mode = MODE_RUN;
     extern char ** environ;
 
-#if (SHADOW_TYPE == SPW_SECUREWARE)
+#if defined(HAVE_GETPRPWNAM) && defined(HAVE_SET_AUTH_PARAMETERS)
     (void) set_auth_parameters(argc, argv);
-#endif /* SPW_SECUREWARE */
+#  ifdef HAVE_INITPRIVS
+    initprivs();
+#  endif
+#endif /* HAVE_GETPRPWNAM && HAVE_SET_AUTH_PARAMETERS */
 
     Argv = argv;
     Argc = argc;
@@ -204,6 +198,18 @@ int main(argc, argv)
 	exit(1);
     }
 
+    /*
+     * Close all file descriptors to make sure we have a nice
+     * clean slate from which to work.  
+     */
+#ifdef HAVE_SYSCONF
+    for (rtn = sysconf(_SC_OPEN_MAX) - 1; rtn > 2; rtn--)
+	(void) close(rtn);
+#else
+    for (rtn = getdtablesize() - 1; rtn > 2; rtn--)
+	(void) close(rtn);
+#endif /* HAVE_SYSCONF */
+
     /*
      * set the prompt based on $SUDO_PROMPT (can be overridden by `-p')
      */
@@ -240,18 +246,6 @@ int main(argc, argv)
     if (cmnd == NULL && NewArgc == 0 && !(sudo_mode & MODE_SHELL))
 	usage(1);
 
-    /*
-     * Close all file descriptors to make sure we have a nice
-     * clean slate from which to work.  
-     */
-#ifdef HAVE_SYSCONF
-    for (rtn = sysconf(_SC_OPEN_MAX) - 1; rtn > 3; rtn--)
-	(void) close(rtn);
-#else
-    for (rtn = getdtablesize() - 1; rtn > 3; rtn--)
-	(void) close(rtn);
-#endif /* HAVE_SYSCONF */
-
     clean_env(environ, badenv_table);
 
     load_globals(sudo_mode);	/* load global variables used throughout sudo */
@@ -265,15 +259,13 @@ int main(argc, argv)
 
 	NewArgv = (char **) malloc (sizeof(char *) * (++NewArgc + 1));
 	if (NewArgv == NULL) {
-	    perror("malloc");
 	    (void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
 	    exit(1);
 	}
 
 	/* add the shell as argv[0] */
 	if (user_shell && *user_shell) {
-	    if ((NewArgv[0] = strrchr(user_shell, '/') + 1) == (char *) 1)
-		NewArgv[0] = user_shell;
+	    NewArgv[0] = user_shell;
 	} else {
 	    (void) fprintf(stderr, "%s: Unable to determine shell.", Argv[0]);
 	    exit(1);
@@ -295,14 +287,13 @@ int main(argc, argv)
 #ifdef SECURE_PATH
     /* replace the PATH envariable with a secure one */
     if (!user_is_exempt() && sudo_setenv("PATH", SECURE_PATH)) {
-	perror("malloc");
 	(void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
 	exit(1);
     }
 #endif /* SECURE_PATH */
 
     if ((sudo_mode & MODE_RUN)) {
-	load_cmnd(sudo_mode);	/* load the cmnd global variable */
+	cmnd_status = load_cmnd(sudo_mode); /* load the cmnd global variable */
     } else if (sudo_mode == MODE_KILL) {
 	remove_timestamp();	/* remove the timestamp ticket file */
 	exit(0);
@@ -310,15 +301,26 @@ int main(argc, argv)
 
     add_env(!(sudo_mode & MODE_SHELL));	/* add in SUDO_* envariables */
 
-    /* validate the user but don't search for "validate" */
+    /* validate the user but don't search for pseudo-commands */
     rtn = validate((sudo_mode != MODE_VALIDATE && sudo_mode != MODE_LIST));
 
     switch (rtn) {
 
 	case VALIDATE_OK:
+	    check_user();
+	    /* fallthrough */
+
 	case VALIDATE_OK_NOPASS:
-	    if (rtn != VALIDATE_OK_NOPASS) 
-		check_user();
+	    /* finally tell the user if the command did not exist */
+	    if (cmnd_status == NOT_FOUND_DOT) {
+		(void) fprintf(stderr, "%s: ignoring `%s' found in '.'\nUse `sudo ./%s' if this is the `%s' you wish to run.\n", Argv[0], cmnd, cmnd, cmnd);
+		exit(1);
+	    } else if (cmnd_status == NOT_FOUND) {
+		(void) fprintf(stderr, "%s: %s: command not found\n", Argv[0],
+		    cmnd);
+		exit(1);
+	    }
+
 	    log_error(ALL_SYSTEMS_GO);
 	    if (sudo_mode == MODE_VALIDATE)
 		exit(0);
@@ -373,9 +375,24 @@ int main(argc, argv)
 	    exit(-1);
 	    break;
 
+	case VALIDATE_NOT_OK:
+	    check_user();
+
+#ifndef DONT_LEAK_PATH_INFO
+	    log_error(rtn);
+	    if (cmnd_status == NOT_FOUND_DOT)
+		(void) fprintf(stderr, "%s: ignoring `%s' found in '.'\nUse `sudo ./%s' if this is the `%s' you wish to run.\n", Argv[0], cmnd, cmnd, cmnd);
+	    else if (cmnd_status == NOT_FOUND)
+		(void) fprintf(stderr, "%s: %s: command not found\n", Argv[0],
+		    cmnd);
+	    else
+		inform_user(rtn);
+	    exit(1);
+	    break;
+#endif /* DONT_LEAK_PATH_INFO */
+
 	default:
 	    log_error(rtn);
-	    set_perms(PERM_FULL_USER, sudo_mode);
 	    inform_user(rtn);
 	    exit(1);
 	    break;
@@ -405,19 +422,16 @@ static void load_globals(sudo_mode)
      * if necesary.  It is assumed that euid is 0 at this point so we
      * can read the shadow passwd file if necesary.
      */
-    user_pw_ent = sudo_getpwuid(getuid());
-    set_perms(PERM_ROOT, sudo_mode);
-    set_perms(PERM_USER, sudo_mode);
-    if (user_pw_ent == NULL) {
+    if ((user_pw_ent = sudo_getpwuid(getuid())) == NULL) {
 	/* need to make a fake user_pw_ent */
-	struct passwd pw_ent;
-	char pw_name[MAX_UID_T_LEN+1];
+	struct passwd pw;
+	char pw_name[MAX_UID_T_LEN + 1];
 
 	/* fill in uid and name fields with the uid */
-	pw_ent.pw_uid = getuid();
-	(void) sprintf(pw_name, "%ld", (long) pw_ent.pw_uid);
-	pw_ent.pw_name = pw_name;
-	user_pw_ent = &pw_ent;
+	pw.pw_uid = getuid();
+	(void) sprintf(pw_name, "%ld", (long) pw.pw_uid);
+	pw.pw_name = pw_name;
+	user_pw_ent = &pw;
 
 	/* complain, log, and die */
 	log_error(GLOBAL_NO_PW_ENT);
@@ -425,6 +439,10 @@ static void load_globals(sudo_mode)
 	exit(1);
     }
 
+    /* Set euid == user and ruid == root */
+    set_perms(PERM_ROOT, sudo_mode);
+    set_perms(PERM_USER, sudo_mode);
+
 #ifdef HAVE_TZSET
     (void) tzset();		/* set the timezone if applicable */
 #endif /* HAVE_TZSET */
@@ -432,20 +450,18 @@ static void load_globals(sudo_mode)
     /*
      * Need to get tty early since it's used for logging
      */
-    if ((tty = (char *) ttyname(0)) || (tty = (char *) ttyname(1))) {
-	if (strncmp(tty, _PATH_DEV, sizeof(_PATH_DEV) - 1) == 0)
-	    tty += sizeof(_PATH_DEV) - 1;
-	if ((tty = (char *) strdup(tty)) == NULL) {
-	    perror("malloc");
+    if ((p = (char *) ttyname(0)) || (p = (char *) ttyname(1))) {
+	if (strncmp(p, _PATH_DEV, sizeof(_PATH_DEV) - 1) == 0)
+	    p += sizeof(_PATH_DEV) - 1;
+	if ((tty = (char *) strdup(p)) == NULL) {
 	    (void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
 	    exit(1);
 	}
-    } else
-	tty = "none";
+    }
 
-#ifdef UMASK
-    (void) umask((mode_t)UMASK);
-#endif /* UMASK */
+#ifdef SUDO_UMASK
+    (void) umask((mode_t)SUDO_UMASK);
+#endif /* SUDO_UMASK */
 
 #ifdef NO_ROOT_SUDO
     if (user_uid == 0) {
@@ -458,11 +474,11 @@ static void load_globals(sudo_mode)
     /*
      * so we know where we are... (do as user)
      */
-    if (!getwd(cwd)) {
+    if (!getcwd(cwd, sizeof(cwd))) {
 	/* try as root... */
 	set_perms(PERM_ROOT, sudo_mode);
-	if (!getwd(cwd)) {
-	    (void) fprintf(stderr, "%s:  Can't get working directory!\n",
+	if (!getcwd(cwd, sizeof(cwd))) {
+	    (void) fprintf(stderr, "%s: Can't get working directory!\n",
 			   Argv[0]);
 	    (void) strcpy(cwd, "unknown");
 	}
@@ -473,7 +489,7 @@ static void load_globals(sudo_mode)
      * load the host global variable from gethostname() and use
      * gethostbyname() if we want to be sure it is fully qualified.
      */
-    if ((gethostname(host, MAXHOSTNAMELEN))) {
+    if ((gethostname(host, sizeof(host)))) {
 	strcpy(host, "localhost");
 	log_error(GLOBAL_NO_HOSTNAME);
 	inform_user(GLOBAL_NO_HOSTNAME);
@@ -493,7 +509,6 @@ static void load_globals(sudo_mode)
     if ((p = strchr(host, '.'))) {
 	*p = '\0';
 	if ((shost = strdup(host)) == NULL) {
-	    perror("malloc");
 	    (void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
 	    exit(1);
 	}
@@ -667,15 +682,14 @@ static void add_env(contiguous)
 	char *to, **from;
 
 	if (contiguous) {
-	    size += (size_t) NewArgv[NewArgc-1] + strlen(NewArgv[NewArgc-1]) -
-		    (size_t) NewArgv[1] + 1;
+	    size += (size_t) (NewArgv[NewArgc-1] - NewArgv[1]) +
+		    strlen(NewArgv[NewArgc-1]) + 1;
 	} else {
 	    for (from = &NewArgv[1]; *from; from++)
 		size += strlen(*from) + 1;
 	}
 
 	if ((buf = (char *) malloc(size)) == NULL) {
-	    perror("malloc");
 	    (void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
 	    exit(1);
 	}
@@ -694,7 +708,6 @@ static void add_env(contiguous)
 	buf = cmnd;
     }
     if (sudo_setenv("SUDO_COMMAND", buf)) {
-	perror("malloc");
 	(void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
 	exit(1);
     }
@@ -711,7 +724,6 @@ static void add_env(contiguous)
 
     /* add the SUDO_USER envariable */
     if (sudo_setenv("SUDO_USER", user_name)) {
-	perror("malloc");
 	(void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
 	exit(1);
     }
@@ -719,7 +731,6 @@ static void add_env(contiguous)
     /* add the SUDO_UID envariable */
     (void) sprintf(idstr, "%ld", (long) user_uid);
     if (sudo_setenv("SUDO_UID", idstr)) {
-	perror("malloc");
 	(void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
 	exit(1);
     }
@@ -727,7 +738,6 @@ static void add_env(contiguous)
     /* add the SUDO_GID envariable */
     (void) sprintf(idstr, "%ld", (long) user_gid);
     if (sudo_setenv("SUDO_GID", idstr)) {
-	perror("malloc");
 	(void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
 	exit(1);
     }
@@ -735,7 +745,6 @@ static void add_env(contiguous)
     /* set PS1 if SUDO_PS1 is set */
     if ((buf = getenv("SUDO_PS1")))
 	if (sudo_setenv("PS1", buf)) {
-	    perror("malloc");
 	    (void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
 	    exit(1);
 	}
@@ -748,12 +757,15 @@ static void add_env(contiguous)
  *  load_cmnd()
  *
  *  This function sets the cmnd global variable
+ *  Returns 1 on success, 0 on failure.
  */
 
-static void load_cmnd(sudo_mode)
+static int load_cmnd(sudo_mode)
     int sudo_mode;
 {
-    if (strlen(NewArgv[0]) > MAXPATHLEN) {
+    int retval;
+
+    if (strlen(NewArgv[0]) >= MAXPATHLEN) {
 	errno = ENAMETOOLONG;
 	(void) fprintf(stderr, "%s: %s: Pathname too long\n", Argv[0],
 		       NewArgv[0]);
@@ -763,11 +775,9 @@ static void load_cmnd(sudo_mode)
     /*
      * Resolve the path
      */
-    if ((cmnd = find_path(NewArgv[0])) == NULL) {
-	(void) fprintf(stderr, "%s: %s: command not found\n", Argv[0],
-		       NewArgv[0]);
-	exit(1);
-    }
+    if ((retval = find_path(NewArgv[0], &cmnd)) != FOUND)
+	cmnd = NewArgv[0];
+    return(retval);
 }
 
 
@@ -783,7 +793,7 @@ static void load_cmnd(sudo_mode)
 static int check_sudoers()
 {
     struct stat statbuf;
-    int fd = -1;
+    int rootstat, i;
     char c;
     int rtn = ALL_SYSTEMS_GO;
 
@@ -792,35 +802,39 @@ static int check_sudoers()
      * Only works if filesystem is readable/writable by root.
      */
     set_perms(PERM_ROOT, 0);
-    if (!lstat(_PATH_SUDO_SUDOERS, &statbuf) && SUDOERS_UID == statbuf.st_uid) {
-	if (SUDOERS_MODE != 0400 && (statbuf.st_mode & 0007777) == 0400) {
-	    if (chmod(_PATH_SUDO_SUDOERS, SUDOERS_MODE) == 0) {
-		(void) fprintf(stderr, "%s: fixed mode on %s\n",
-		    Argv[0], _PATH_SUDO_SUDOERS);
-		if (statbuf.st_gid != SUDOERS_GID) {
-		    if (!chown(_PATH_SUDO_SUDOERS,GID_NO_CHANGE,SUDOERS_GID)) {
-			(void) fprintf(stderr, "%s: set group on %s\n",
-			    Argv[0], _PATH_SUDO_SUDOERS);
-			statbuf.st_gid = SUDOERS_GID;
-		    } else {
-			(void) fprintf(stderr,"%s: Unable to set group on %s: ",
-			    Argv[0], _PATH_SUDO_SUDOERS);
-			perror("");
-		    }
+    if ((rootstat = lstat(_PATH_SUDO_SUDOERS, &statbuf)) == 0 &&
+	SUDOERS_UID == statbuf.st_uid && SUDOERS_MODE != 0400 &&
+	(statbuf.st_mode & 0007777) == 0400) {
+
+	if (chmod(_PATH_SUDO_SUDOERS, SUDOERS_MODE) == 0) {
+	    (void) fprintf(stderr, "%s: fixed mode on %s\n",
+		Argv[0], _PATH_SUDO_SUDOERS);
+	    if (statbuf.st_gid != SUDOERS_GID) {
+		if (!chown(_PATH_SUDO_SUDOERS,GID_NO_CHANGE,SUDOERS_GID)) {
+		    (void) fprintf(stderr, "%s: set group on %s\n",
+			Argv[0], _PATH_SUDO_SUDOERS);
+		    statbuf.st_gid = SUDOERS_GID;
+		} else {
+		    (void) fprintf(stderr,"%s: Unable to set group on %s: ",
+			Argv[0], _PATH_SUDO_SUDOERS);
+		    perror("");
 		}
-	    } else {
-		(void) fprintf(stderr, "%s: Unable to fix mode on %s: ",
-		    Argv[0], _PATH_SUDO_SUDOERS);
-		perror("");
 	    }
+	} else {
+	    (void) fprintf(stderr, "%s: Unable to fix mode on %s: ",
+		Argv[0], _PATH_SUDO_SUDOERS);
+	    perror("");
 	}
     }
 
+    /*
+     * Sanity checks on sudoers file.  Must be done as sudoers
+     * file owner.  We already did a stat as root, so use that
+     * data if we can't stat as sudoers file owner.
+     */
     set_perms(PERM_SUDOERS, 0);
 
-    if ((fd = open(_PATH_SUDO_SUDOERS, O_RDONLY)) < 0 || read(fd, &c, 1) == -1)
-	rtn = NO_SUDOERS_FILE;
-    else if (lstat(_PATH_SUDO_SUDOERS, &statbuf))
+    if (lstat(_PATH_SUDO_SUDOERS, &statbuf) != 0 && rootstat != 0)
 	rtn = NO_SUDOERS_FILE;
     else if (!S_ISREG(statbuf.st_mode))
 	rtn = SUDOERS_NOT_FILE;
@@ -828,9 +842,25 @@ static int check_sudoers()
 	rtn = SUDOERS_WRONG_MODE;
     else if (statbuf.st_uid != SUDOERS_UID || statbuf.st_gid != SUDOERS_GID)
 	rtn = SUDOERS_WRONG_OWNER;
-
-    if (fd != -1)
-	(void) close(fd);
+    else {
+	/* Solaris sometimes returns EAGAIN so try 10 times */
+	for (i = 0; i < 10 ; i++) {
+	    errno = 0;
+	    if ((sudoers_fp = fopen(_PATH_SUDO_SUDOERS, "r")) == NULL ||
+		fread(&c, sizeof(c), 1, sudoers_fp) != 1) {
+		sudoers_fp = NULL;
+		if (errno != EAGAIN && errno != EWOULDBLOCK)
+		    break;
+	    } else
+		break;
+	    sleep(1);
+	}
+	if (sudoers_fp == NULL) {
+	    fprintf(stderr, "%s: cannot open %s: ", Argv[0], _PATH_SUDO_SUDOERS);
+	    perror("");
+	    rtn = NO_SUDOERS_FILE;
+	}
+    }
 
     set_perms(PERM_ROOT, 0);
     set_perms(PERM_USER, 0);
@@ -851,7 +881,7 @@ void set_perms(perm, sudo_mode)
     int perm;
     int sudo_mode;
 {
-    struct passwd *pw_ent;
+    struct passwd *pw;
 
     switch (perm) {
 	case PERM_ROOT:
@@ -900,30 +930,52 @@ void set_perms(perm, sudo_mode)
 					exit(1);
 				    }
 				} else {
-				    if (!(pw_ent = getpwnam(runas_user))) {
+				    if (!(pw = getpwnam(runas_user))) {
 					(void) fprintf(stderr,
 					    "%s: no passwd entry for %s!\n",
 					    Argv[0], runas_user);
 					exit(1);
 				    }
 
-				    if (setgid(pw_ent->pw_gid)) {
+				    /* Set $USER to match target user */
+				    if (sudo_setenv("USER", pw->pw_name)) {
+					(void) fprintf(stderr,
+					    "%s: cannot allocate memory!\n",
+					    Argv[0]);
+					exit(1);
+				    }
+
+				    if (setgid(pw->pw_gid)) {
 					(void) fprintf(stderr,
 					    "%s: cannot set gid to %d: ",  
-					    Argv[0], pw_ent->pw_gid);
+					    Argv[0], pw->pw_gid);
 					perror("");
 					exit(1);
 				    }
 
-				    if (setuid(pw_ent->pw_uid)) {
+				    /*
+				     * Initialize group vector only if are
+				     * going to run as a non-root user.
+				     */
+				    if (strcmp(runas_user, "root") != 0 &&
+					initgroups(runas_user, pw->pw_gid)
+					== -1) {
+					(void) fprintf(stderr,
+					    "%s: cannot set group vector ",
+					    Argv[0]);
+					perror("");
+					exit(1);
+				    }
+
+				    if (setuid(pw->pw_uid)) {
 					(void) fprintf(stderr,
 					    "%s: cannot set uid to %d: ",  
-					    Argv[0], pw_ent->pw_uid);
+					    Argv[0], pw->pw_uid);
 					perror("");
 					exit(1);
 				    }
 				    if (sudo_mode & MODE_RESET_HOME)
-					runas_homedir = pw_ent->pw_dir;
+					runas_homedir = pw->pw_dir;
 				}
 
 				break;
