@@ -1,5 +1,5 @@
 /*
- *  CU sudo version 1.5.3
+ *  CU sudo version 1.5.7
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,13 +24,12 @@
  *  Chris Jepeway <jepeway@cs.utk.edu>
  */
 
-#ifndef lint
-static char rcsid[] = "$Id: testsudoers.c,v 1.30 1996/11/14 02:37:16 millert Exp $";
-#endif /* lint */
-
 #include "config.h"
 
 #include <stdio.h>
+#ifdef STDC_HEADERS
+#  include <stdlib.h>
+#endif /* STDC_HEADERS */
 #ifdef HAVE_UNISTD_H
 #  include <unistd.h>
 #endif /* HAVE_UNISTD_H */
@@ -65,6 +64,10 @@ static char rcsid[] = "$Id: testsudoers.c,v 1.30 1996/11/14 02:37:16 millert Exp
 
 #include "sudo.h"
 
+#ifndef lint
+static const char rcsid[] = "$Id: testsudoers.c,v 1.45 1998/11/18 04:16:13 millert Exp $";
+#endif /* lint */
+
 /*
  * Globals
  */
@@ -75,10 +78,10 @@ extern int num_interfaces;
 
 char *cmnd = NULL;
 char *cmnd_args = NULL;
-char *runas_user = "root";
-char host[MAXHOSTNAMELEN+1];
+char *runas_user = RUNAS_DEFAULT;
+char host[MAXHOSTNAMELEN];
 char *shost;
-char cwd[MAXPATHLEN+1];
+char cwd[MAXPATHLEN];
 struct passwd *user_pw_ent;
 char **Argv, **NewArgv;
 int  Argc, NewArgc;
@@ -181,23 +184,23 @@ int usergr_matches(group, user)
     char *group;
     char *user;
 {
-    struct group *grpent;
+    struct group *grp;
     char **cur;
 
     /* make sure we have a valid usergroup, sudo style */
     if (*group++ != '%')
 	return(FALSE);
 
-    if ((grpent = getgrnam(group)) == NULL) 
+    if ((grp = getgrnam(group)) == NULL) 
 	return(FALSE);
 
     /*
      * Check against user's real gid as well as group's user list
      */
-    if (getgid() == grpent->gr_gid)
+    if (getgid() == grp->gr_gid)
 	return(TRUE);
 
-    for (cur=grpent->gr_mem; *cur; cur++) {
+    for (cur=grp->gr_mem; *cur; cur++) {
 	if (strcmp(*cur, user) == 0)
 	    return(TRUE);
     }
@@ -224,13 +227,12 @@ int netgr_matches(netgr, host, user)
 #ifdef HAVE_GETDOMAINNAME
     /* get the domain name (if any) */
     if (domain == (char *) -1) {
-	if ((domain = (char *) malloc(MAXHOSTNAMELEN + 1)) == NULL) {
-	    perror("malloc");
+	if ((domain = (char *) malloc(MAXHOSTNAMELEN)) == NULL) {
 	    (void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
 	    exit(1);
 	}
 
-	if (getdomainname(domain, MAXHOSTNAMELEN + 1) != 0 || *domain == '\0') {
+	if (getdomainname(domain, MAXHOSTNAMELEN) != 0 || *domain == '\0') {
 	    (void) free(domain);
 	    domain = NULL;
 	}
@@ -256,7 +258,7 @@ int main(argc, argv)
     int argc;
     char **argv;
 {
-    struct passwd pw_ent;
+    struct passwd pw;
     char *p;
 #ifdef	YYDEBUG
     extern int yydebug;
@@ -268,14 +270,14 @@ int main(argc, argv)
 
     if (Argc >= 6 && strcmp(Argv[1], "-u") == 0) {
 	runas_user = Argv[2];
-	pw_ent.pw_name = Argv[3];
+	pw.pw_name = Argv[3];
 	strcpy(host, Argv[4]);
 	cmnd = Argv[5];
 
 	NewArgv = &Argv[5];
 	NewArgc = Argc - 5;
     } else if (Argc >= 4) {
-	pw_ent.pw_name = Argv[1];
+	pw.pw_name = Argv[1];
 	strcpy(host, Argv[2]);
 	cmnd = Argv[3];
 
@@ -286,12 +288,11 @@ int main(argc, argv)
 	exit(1);
     }
 
-    user_pw_ent = &pw_ent;		/* need user_pw_ent->pw_name defined */
+    user_pw_ent = &pw;		/* need user_pw_ent->pw_name defined */
 
     if ((p = strchr(host, '.'))) {
 	*p = '\0';
 	if ((shost = strdup(host)) == NULL) {
-	    perror("malloc");
 	    (void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
 	    exit(1);
 	}
@@ -308,7 +309,6 @@ int main(argc, argv)
 	size = (size_t) NewArgv[NewArgc-1] + strlen(NewArgv[NewArgc-1]) -
 	       (size_t) NewArgv[1] + 1;
 	if ((cmnd_args = (char *) malloc(size)) == NULL) {
-	    perror("malloc");
 	    (void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);  
 	    exit(1);
 	}
@@ -333,7 +333,7 @@ int main(argc, argv)
     } else {
 	(void) printf("parses OK.\n\n");
 	if (top == 0)
-	    (void) printf("User %s not found\n", pw_ent.pw_name);
+	    (void) printf("User %s not found\n", pw.pw_name);
 	else while (top) {
 	    (void) printf("[%d]\n", top-1);
 	    (void) printf("user_match : %d\n", user_matches);
