@@ -1,5 +1,5 @@
 /*
- *  CU sudo version 1.5.3
+ *  CU sudo version 1.5.7
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,13 +26,12 @@
  *  Todd C. Miller  Sun Jun  5 17:22:31 MDT 1994
  */
 
-#ifndef lint
-static char rcsid[] = "$Id: tgetpass.c,v 1.50 1996/11/14 02:37:16 millert Exp $";
-#endif /* lint */
-
 #include "config.h"
 
 #include <stdio.h>
+#ifdef STDC_HEADERS
+#include <stdlib.h>
+#endif /* STDC_HEADERS */
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
@@ -44,6 +43,7 @@ static char rcsid[] = "$Id: tgetpass.c,v 1.50 1996/11/14 02:37:16 millert Exp $"
 #endif /* HAVE_STRINGS_H */
 #include <limits.h>
 #include <pwd.h>
+#include <sys/param.h>
 #include <sys/types.h>
 #ifdef HAVE_SYS_BSDTYPES_H
 #include <sys/bsdtypes.h>
@@ -52,6 +52,7 @@ static char rcsid[] = "$Id: tgetpass.c,v 1.50 1996/11/14 02:37:16 millert Exp $"
 #include <sys/select.h>
 #endif /* HAVE_SYS_SELECT_H */
 #include <sys/time.h>
+#include <errno.h>
 #include <signal.h>
 #include <fcntl.h>
 #ifdef HAVE_TERMIOS_H
@@ -64,14 +65,14 @@ static char rcsid[] = "$Id: tgetpass.c,v 1.50 1996/11/14 02:37:16 millert Exp $"
 #include <sys/ioctl.h>
 #endif /* HAVE_TERMIO_H */
 #endif /* HAVE_TERMIOS_H */
-#if (SHADOW_TYPE == SPW_SECUREWARE)
+#ifdef HAVE_GETPRPWNAM
 #  ifdef __hpux
 #    include <hpsecurity.h>
 #  else
 #    include <sys/security.h>
-#  endif /* __hpux */ 
-#  include <prot.h>
-#endif /* SPW_SECUREWARE */
+#  endif /* __hpux */
+#  include <prot.h>			/* for AUTH_MAX_PASSWD_LENGTH */
+#endif /* HAVE_GETPRPWNAM */
 
 #include <pathnames.h>
 #include "compat.h"
@@ -80,6 +81,10 @@ static char rcsid[] = "$Id: tgetpass.c,v 1.50 1996/11/14 02:37:16 millert Exp $"
 #define TCSASOFT	0
 #endif /* TCSASOFT */
 
+#ifndef lint
+static const char rcsid[] = "$Id: tgetpass.c,v 1.72 1998/11/18 04:16:13 millert Exp $";
+#endif /* lint */
+
 
 /******************************************************************
  *
@@ -90,11 +95,9 @@ static char rcsid[] = "$Id: tgetpass.c,v 1.50 1996/11/14 02:37:16 millert Exp $"
  *  and input will time out based on the value of timeout.
  */
 
-char * tgetpass(prompt, timeout, user, host)
+char * tgetpass(prompt, timeout)
     const char *prompt;
     int timeout;
-    char *user;
-    char *host;
 {
 #ifdef HAVE_TERMIOS_H
     struct termios term;
@@ -114,9 +117,8 @@ char * tgetpass(prompt, timeout, user, host)
     int n, echo;
     FILE *input, *output;
     static char buf[_PASSWD_LEN + 1];
-    fd_set readfds;
+    fd_set *readfds;
     struct timeval tv;
-    char *p;
 
     /*
      * mask out SIGINT and SIGTSTP, should probably just catch and deal.
@@ -137,9 +139,19 @@ char * tgetpass(prompt, timeout, user, host)
     if ((input = fopen(_PATH_TTY, "r+")) == NULL) {
 	input = stdin;
 	output = stderr;
-	(void) fflush(output);
     } else {
 	output = input;
+	setbuf(output, NULL);
+    }
+
+    /* print the prompt */
+    if (prompt)
+	fputs(prompt, output);
+
+    /* rewind if necesary */
+    if (input == output) {
+	(void) fflush(output);
+	(void) rewind(output);
     }
 
     /*
@@ -167,61 +179,37 @@ char * tgetpass(prompt, timeout, user, host)
 #endif /* HAVE_TERMIO_H */
 #endif /* HAVE_TERMIOS_H */
 
-    /* print the prompt */
-    if (prompt) {
-	p = (char *) prompt;
-	do {
-	    /* expand %u -> username, %h -> host */
-	    switch (*p) {
-		case '%':   if (user && *(p+1) == 'u') {
-				(void) fputs(user, output);
-				p++;
-				break;
-			    } else if (host && *(p+1) == 'h') {
-				(void) fputs(host, output);
-				p++;
-				break;
-			    }
-
-		default:    (void) fputc(*p, output);
-	    }
-	} while (*(++p));
-    }
-
-    /* rewind if necesary */
-    if (input == output) {
-	(void) fflush(output);
-	(void) rewind(output);
-    }
-
     /*
      * Timeout of <= 0 means no timeout
      */
     if (timeout > 0) {
 	/* setup for select(2) */
-	FD_ZERO(&readfds);
-	FD_SET(fileno(input), &readfds);
+	n = howmany(fileno(input) + 1, NFDBITS) * sizeof(fd_mask);
+	if ((readfds = (fd_set *) malloc(n)) == NULL) {
+	    (void) fprintf(stderr, "Cannot allocate memory: ");
+	    perror("");
+	    return(NULL);
+	}
+	(void) memset((VOID *)readfds, 0, n);
+	FD_SET(fileno(input), readfds);
 
 	/* set timeout for select */
 	tv.tv_sec = timeout;
 	tv.tv_usec = 0;
 
-	/* how many file descriptors may we have? */
-#ifdef HAVE_SYSCONF
-	n = sysconf(_SC_OPEN_MAX);
-#else
-	n = getdtablesize();
-#endif /* HAVE_SYSCONF */
-
 	/*
 	 * get password or return empty string if nothing to read by timeout
 	 */
 	buf[0] = '\0';
-	if (select(n, &readfds, 0, 0, &tv) > 0 && fgets(buf, sizeof(buf), input)) {
+	while ((n = select(fileno(input) + 1, readfds, 0, 0, &tv)) == -1 &&
+	    errno == EINTR)
+	    ;
+	if (n != 0 && fgets(buf, sizeof(buf), input)) {
 	    n = strlen(buf);
 	    if (buf[n - 1] == '\n')
 		buf[n - 1] = '\0';
 	}
+	(void) free(readfds);
     } else {
 	buf[0] = '\0';
 	if (fgets(buf, sizeof(buf), input)) {
