@@ -1,5 +1,5 @@
 /*
- *  CU sudo version 1.5.3
+ *  CU sudo version 1.5.7
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,10 +25,6 @@
  *  Todd C. Miller (millert@colorado.edu) Sat Mar 25 21:50:36 MST 1995
  */
 
-#ifndef lint
-static char rcsid[] = "$Id: visudo.c,v 1.80 1996/11/14 02:37:16 millert Exp $";
-#endif /* lint */
-
 #include "config.h"
 
 #include <stdio.h>
@@ -59,7 +55,6 @@ static char rcsid[] = "$Id: visudo.c,v 1.80 1996/11/14 02:37:16 millert Exp $";
 #include <netinet/in.h>
 
 #include "sudo.h"
-#include <options.h>
 #include "version.h"
 
 #ifndef STDC_HEADERS
@@ -74,6 +69,10 @@ extern int stat		__P((const char *, struct stat *));
 #define SA_RESETHAND    0
 #endif /* POSIX_SIGNALS && !SA_RESETHAND */
 
+#ifndef lint
+static const char rcsid[] = "$Id: visudo.c,v 1.95 1998/11/18 04:16:13 millert Exp $";
+#endif /* lint */
+
 /*
  * Function prototypes
  */
@@ -105,7 +104,7 @@ int NewArgc = 0;
 char *sudoers = _PATH_SUDO_SUDOERS;
 char *stmp = _PATH_SUDO_STMP;
 int parse_error = FALSE;
-char *runas_user = "root";
+char *runas_user = RUNAS_DEFAULT;
 
 /*
  * For the parsing routines
@@ -145,18 +144,24 @@ int main(argc, argv)
      * If passesd -V then print version, else print usage
      * if any other option...
      */
-    if (argc == 2)
+    if (argc == 2) {
 	if (!strcmp(Argv[1], "-V")) {
 	    (void) printf("visudo version %s\n", version);
 	    exit(0);
 	} else {
 	    usage();
 	}
-    else if (argc != 1)
+    } else if (argc != 1) {
 	usage();
+    }
 
     /* user_pw_ent needs to point to something... */
-    user_pw_ent = getpwuid(getuid());
+    if ((user_pw_ent = getpwuid(getuid())) == NULL) {
+	(void) fprintf(stderr, "%s: Can't find you in the passwd database: ",
+	    Argv[0]);
+	perror(stmp);
+	exit(1);
+    }
 
 #ifdef ENV_EDITOR
     /*
@@ -217,7 +222,7 @@ int main(argc, argv)
 	 */
 	if (strlen(Editor) + strlen(stmp) + 30 > sizeof(buf)) {
 	    (void) fprintf(stderr, "%s: Buffer too short (line %d).\n",
-			   __LINE__, Argv[0]);
+			   Argv[0], __LINE__);
 	    Exit(-1);
 	}
 	if (parse_error == TRUE)
@@ -269,7 +274,7 @@ int main(argc, argv)
 	    if (yyparse() && parse_error != TRUE) {
 		(void) fprintf(stderr,
 		    "%s: Failed to parse temporary file (%s), unknown error.\n",
-		    Argv[0], stmp, sudoers);
+		    Argv[0], stmp);
 		parse_error = TRUE;
 	    }
 	} else {
@@ -315,7 +320,7 @@ int main(argc, argv)
      * rename(2)'d to sudoers.  If the rename(2) fails we try using
      * mv(1) in case stmp and sudoers are on different filesystems.
      */
-    if (rename(stmp, sudoers))
+    if (rename(stmp, sudoers)) {
 	if (errno == EXDEV) {
 	    char *tmpbuf;
 
@@ -349,6 +354,7 @@ int main(argc, argv)
 	    perror("");
 	    Exit(-1);
 	}
+    }
 
     return(0);
 }
